
/* Web Polygraph       http://www.web-polygraph.org/
 * (C) 2003-2006 The Measurement Factory
 * Licensed under the Apache License, Version 2.0 */

#ifndef POLYGRAPH__SERVER_FTPSRVXACT_H
#define POLYGRAPH__SERVER_FTPSRVXACT_H

#include "runtime/FtpMsg.h"
#include "server/SrvXact.h"

class FtpSrvXact: public SrvXact {
	public:
		FtpSrvXact();

		virtual void reset();
		virtual void finish(Error err);

		virtual void noteReadReady(int fd);
		virtual void noteWriteReady(int fd);

	protected:
		virtual void doStart();
		virtual void noteConnReady();

		virtual void noteHdrDataReady();
		virtual void noteRepSent();
		virtual void makeRep(WrBuf &buf);

		Error interpretCmd();
		Error startListenDataChannel();
		void stopListenDataChannel();
		void stopDataChannel();
		void acceptDataConnection();
		void noteDataChannelWriteReady();
		Error processRetr();
		bool prepareContent();

		void makeSimpleResponse(ostream &os, const String &reply);
		void makeMdtmResponse(ostream &os);
		void makeSizeResponse(ostream &os);
		void makePasvResponse(ostream &os);
		void makeRetrResponse(ostream &os);

		FtpReq theReq;
		String theDir; // current directory, controlled by CWD commands
		String theFilename; // last RETR command argument
		Socket theSock; // data channel listening socket
		FileScanReserv theReserv;
		MessageSize theWrittenSize; // accumulates, then updates theRepSize
		Connection theDataConn; // data connection
		int theDataPort; // port to listen for incoming data connections

		enum DataChannelState {
			dcsNone,
			dcsNoConnection,
			dcsInProgress,
			dcsAborted,
			dcsDone
		};
		DataChannelState theDataChannelState;
};

#endif
